/***************************************************************************
 * blitz/array/slice.h    Helper classes for slicing arrays
 *
 * $Id: slice.h,v 1.2 2001/01/24 20:22:51 tveldhui Exp $
 *
 * Copyright (C) 1997-2001 Todd Veldhuizen <tveldhui@oonumerics.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Suggestions:          blitz-dev@oonumerics.org
 * Bugs:                 blitz-bugs@oonumerics.org
 *
 * For more information, please see the Blitz++ Home Page:
 *    http://oonumerics.org/blitz/
 *
 ***************************************************************************
 * $Log: slice.h,v $
 *
 * Revision 1.3  2002/10/26 15:53:00 mdsuresh
 * implemented _xYYYYY classes  
 *
 * Revision 1.2  2001/01/24 20:22:51  tveldhui
 * Updated copyright date in headers.
 *
 * Revision 1.1.1.1  2000/06/19 12:26:14  tveldhui
 * Imported sources
 *
 * Revision 1.2  1998/03/14 00:04:47  tveldhui
 * 0.2-alpha-05
 *
 * Revision 1.1  1997/07/16 14:51:20  tveldhui
 * Update: Alpha release 0.2 (Arrays)
 *
 */

#ifndef BZ_ARRAYSLICE_H
#define BZ_ARRAYSLICE_H

#ifndef BZ_ARRAY_H
 #error <blitz/array/slice.h> must be included via <blitz/array.h>
#endif

BZ_NAMESPACE(blitz)

// Forward declaration

template<class T, int N>
class Array;
/*
template<int a, int b> 
class sliceMax {
   enum { 
       max = a > b ? a:b
   }
};

template<int a, int b> 
class sliceMin {
   enum { 
       max = a < b ? a:b
   }
};
*/



class nilArraySection { };

template<class T>
class ArraySectionInfo {
public:
    enum { isValidType = 0, rank = 0, isPick = 0 };
};

template<>
class ArraySectionInfo<Range> {
public:
    enum { isValidType = 1, rank = 1, isPick = 0 };
};

template<>
class ArraySectionInfo<int> {
public:
    enum { isValidType = 1, rank = 0, isPick = 0 };
};

template<>
class ArraySectionInfo<nilArraySection> {
public:
    enum { isValidType = 1, rank = 0, isPick = 0 };
};

template< class T_numtype, class T1, class T2 = nilArraySection, 
    class T3 = nilArraySection, class T4 = nilArraySection, 
    class T5 = nilArraySection, class T6 = nilArraySection, 
    class T7 = nilArraySection, class T8 = nilArraySection, 
    class T9 = nilArraySection, class T10 = nilArraySection, 
    class T11 = nilArraySection>
class SliceInfo {
public:
    enum { 
        numValidTypes = ArraySectionInfo<T1>::isValidType
                      + ArraySectionInfo<T2>::isValidType
                      + ArraySectionInfo<T3>::isValidType
                      + ArraySectionInfo<T4>::isValidType
                      + ArraySectionInfo<T5>::isValidType
                      + ArraySectionInfo<T6>::isValidType
                      + ArraySectionInfo<T7>::isValidType
                      + ArraySectionInfo<T8>::isValidType
                      + ArraySectionInfo<T9>::isValidType
                      + ArraySectionInfo<T10>::isValidType
                      + ArraySectionInfo<T11>::isValidType,

        rank          =            ArraySectionInfo<T1>::rank
                                     + ArraySectionInfo<T2>::rank
                                     + ArraySectionInfo<T3>::rank
                                     + ArraySectionInfo<T4>::rank
                                     + ArraySectionInfo<T5>::rank
                                     + ArraySectionInfo<T6>::rank
                                     + ArraySectionInfo<T7>::rank
                                     + ArraySectionInfo<T8>::rank
                                     + ArraySectionInfo<T9>::rank
                                     + ArraySectionInfo<T10>::rank
                                     + ArraySectionInfo<T11>::rank,

        isPick        = ArraySectionInfo<T1>::isPick
                      + ArraySectionInfo<T2>::isPick
                      + ArraySectionInfo<T3>::isPick
                      + ArraySectionInfo<T4>::isPick
                      + ArraySectionInfo<T5>::isPick
                      + ArraySectionInfo<T6>::isPick
                      + ArraySectionInfo<T7>::isPick
                      + ArraySectionInfo<T8>::isPick
                      + ArraySectionInfo<T9>::isPick
                      + ArraySectionInfo<T10>::isPick
                      + ArraySectionInfo<T11>::isPick
        };

    typedef Array<T_numtype,numValidTypes> T_array;
    typedef Array<T_numtype,rank> T_slice;
};
/* mdsuresh: */
/* Extended ArraySection implementation  */
/* These classes use reverse conventions  to accomplish slicing */
/* Overtime, expecting to replace all usage of ArraySectionInfo with _xArraySectionInfo */
/* Example of indirect Usage: */
/* Array<int,3> A(4, 4, 4); */
/* A(4, Range(2,4) )  is same as A(4, Range(2,4), Range:all() ) */

template<int _rank>
class _xNilArraySection { 
     enum { rank = _rank
      };
};

template<class T>
class _xArraySectionInfo {
public:
    enum { isValidType = -1 , rank = 0, isPick = 0 };
};

template<>
class _xArraySectionInfo<Range> {
public:
    enum { isValidType = 0, rank = 0, isPick = 0 };
};

template<>
class _xArraySectionInfo<int> {
public:
    enum { isValidType = 0, rank = -1, isPick = 0 };
};

template<int n>
class _xArraySectionInfo< _xNilArraySection<n> > {
public:
    enum { isValidType = 0, rank = 0, isPick = 0 };
};

template< int  currentRank, class T_numtype, class T1, class T2 = _xNilArraySection<1>, 
          class T3 = _xNilArraySection<2>, class T4 = _xNilArraySection<3>, 
          class T5 = _xNilArraySection<4>, class T6 = _xNilArraySection<5>,
          class T7 = _xNilArraySection<6>, class T8 = _xNilArraySection<7>, 
          class T9 = _xNilArraySection<8>, class T10 = _xNilArraySection<9>, 
          class T11 = _xNilArraySection<10> >
class _xSliceInfo {
public:
    enum { 
        numValidTypes = currentRank + _xArraySectionInfo<T1>::isValidType
                                    + _xArraySectionInfo<T2>::isValidType
                                    + _xArraySectionInfo<T3>::isValidType
                                    + _xArraySectionInfo<T4>::isValidType
                                    + _xArraySectionInfo<T5>::isValidType
                                    + _xArraySectionInfo<T6>::isValidType
                                    + _xArraySectionInfo<T7>::isValidType
                                    + _xArraySectionInfo<T8>::isValidType
                                    + _xArraySectionInfo<T9>::isValidType
                                    + _xArraySectionInfo<T10>::isValidType
                                    + _xArraySectionInfo<T11>::isValidType 
                      ,

        rank          = currentRank  + _xArraySectionInfo<T1>::rank
                                     + _xArraySectionInfo<T2>::rank
                                     + _xArraySectionInfo<T3>::rank
                                     + _xArraySectionInfo<T4>::rank
                                     + _xArraySectionInfo<T5>::rank
                                     + _xArraySectionInfo<T6>::rank
                                     + _xArraySectionInfo<T7>::rank
                                     + _xArraySectionInfo<T8>::rank
                                     + _xArraySectionInfo<T9>::rank
                                     + _xArraySectionInfo<T10>::rank
                                     + _xArraySectionInfo<T11>::rank,

        isPick        = _xArraySectionInfo<T1>::isPick
                      + _xArraySectionInfo<T2>::isPick
                      + _xArraySectionInfo<T3>::isPick
                      + _xArraySectionInfo<T4>::isPick
                      + _xArraySectionInfo<T5>::isPick
                      + _xArraySectionInfo<T6>::isPick
                      + _xArraySectionInfo<T7>::isPick
                      + _xArraySectionInfo<T8>::isPick
                      + _xArraySectionInfo<T9>::isPick
                      + _xArraySectionInfo<T10>::isPick
                      + _xArraySectionInfo<T11>::isPick
        };

    typedef Array<T_numtype,numValidTypes> T_array;
    typedef Array<T_numtype,rank> T_slice;
};



BZ_NAMESPACE_END

#endif // BZ_ARRAYSLICE_H
