/***************************************************************************
 * blitz/lhsrhs.h    Definition of the lhs rhs routines
 *
 * $Id: lhsrhs.h,v 1.10 2002/10/24 22:12:49 mdsuresh Exp $
 *
 * Copyright (C) 2002 Suresh kumar Devanathan <mdsuresh@yahoo.com>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * Suggestions:          blitz-dev@oonumerics.org
 * Bugs:                 blitz-bugs@oonumerics.org
 *
 * For more information, please see the Blitz++ Home Page:
 *    http://oonumerics.org/blitz/
 *
 ***************************************************************************
 * $Log: lhsrhs.h,v $
 * Revision 1.0
 * implemented lhs, rhs, Nil, ref, out
 */
 
#ifndef _BZ_NAMESPACE_LHSRHS
#define _BZ_NAMESPACE_LHSRHS
#include <iostream>


 
BZ_NAMESPACE(blitz)

   struct Nil 
   {
	   Nil() {
	   }
	   
	   
	};
	
	
#ifdef _BZ_DEBUG
   inline std::ostream&	operator <<(std::ostream& _out,	Nil& )
	{
	   return _out;
	}
#endif
	

	template<class T1, 
			 class T2 =	Nil,
			 class T3 =	Nil, 
			 class T4 =	Nil,
			 class T5 =	Nil,
			 class T6 =	Nil>
	struct _rhs	
	{
	   _rhs(T1 _x1,
		   T2 _x2 =	T2(),
		   T3 _x3 =	T3(),
		   T4 _x4 =	T4(),
		   T5 _x5 =	T5(),
		   T6 _x6 =	T6()):
			x1(_x1),
			x2(_x2),
			x3(_x3),
			x4(_x4),
			x5(_x5),
			x6(_x6)
		  {
			  //std::cout << _x1 <<	_x2	<< _x3 << _x4 << _x5 <<	_x6;
		  }
		  
	   
		 T1	x1;
		 T2	x2;
		 T3	x3;
		 T4	x4;
		 T5	x5;
		 T6	x6;
		   
	};
	
	/* 1 Parameter */
	template<
	  class	T1>
	   _rhs<T1>
				   rhs(	T1 x1)
					{
					  return _rhs<T1>(x1);
					}
	/* 2 Parameters	*/
	  template<
	  class	T1,
	  class	T2>
	   _rhs<T1,	T2>
				   rhs(	T1 x1,
						T2 x2)
					{
					  return _rhs<T1,T2>(x1,x2);
					}
					
						  
	/* 3 Parameters	*/
	  template<
	  class	T1,
	  class	T2,
	  class	T3>
	   _rhs<T1,	T2,	T3>
				   rhs(	T1 x1,
						T2 x2,
						T3 x3)
					{
					  return _rhs<T1,T2,T3>(x1,x2, x3);
					}
					
	  /* 4 Parameters */
	  template<
	  class	T1,
	  class	T2,
	  class	T3,
	  class	T4>
	   _rhs<T1,	T2,	T3,	T4>
				   rhs(	T1 x1,
						T2 x2,
						T3 x3,
						T4 x4)
					{
					  return _rhs<T1,T2,T3,T4>(x1,x2, x3, x4);
					}
	  /* 5 Parameters */
	  template<
	  class	T1,
	  class	T2,
	  class	T3,
	  class	T4,
	  class	T5>
	   _rhs<T1,	T2,	T3,	T4,	T5>
				   rhs(	T1 x1,
						T2 x2,
						T3 x3,
						T4 x4,
						T5 x5)
					{
					  return _rhs<T1,T2,T3,T4,T5>(x1,x2, x3, x4, x5);
					}
					
					
	  /* 6 Parameters */
	  template<
	  class	T1,
	  class	T2,
	  class	T3,
	  class	T4,
	  class	T5,
	  class	T6>
	   _rhs<T1,	T2,	T3,	T4,	T5,	T6>
				   rhs(	T1 x1, 
						T2 x2, 
						T3 x3, 
						T4 x4, 
						T5 x5, 
						T6 x6)
					{
					  return _rhs<T1,T2,T3,T4,T5,T6>(x1,x2,x3,x4,x5,x6);
					}
					
					
  
		
	template<class T>
	   struct _reference
	   {
		 inline	_reference(const T & _m) : m( (T&) _m)
		 {
		 }
		 
		 inline	_reference(const _reference& _r) : m(_r.m)
		 {
		 
		 }
		 T&	m;
		 
		 template<class	_T>
		 inline	_T operator	=(_T rhs) 
		 {
			 m = rhs;
			 return	 rhs;
		 }
		 
	   
	   };
	   
		
	   
		template< class	T ,	bool _adelete>
		  struct _out
		  {
			  inline _out(T& _x) : x(_x)
			  {
			   
			  }
			  
			  inline void free()
			  {
				if(_adelete) delete	this;
			  }
			  
			  template<class _T>
			  inline _T	operator=(_T rhs)
			  {
				 x.reference(rhs);
				 free();
				 return	rhs;
			  }
		  
			  T& x;
			  
			
		 };
		 
		  template<	class T	, bool _adelete>
		  struct _pointer
		  {
			  inline _pointer(T& _x) : x(_x)
			  {
			   
			  }
			  
			  inline void free()
			  {
				if(_adelete) delete	this;
			  }
			  
			  template<class _T>
			  inline _T	operator=(_T rhs)
			  {
				 x = rhs;
				 free();
				 return	rhs;
			  }
		  
			  T& x;
			  
			
		 };
		 
		 
	template<class T>
	struct _reference< _out<T, false> >
	{
		  _reference(const	_out<T,false>& _m):	 m(_m) 
		  {
		  }
		  _reference(const _reference&	_rm	) :	m(_rm.m)
		  {
		  }
		  _out<T,false>	 m;
		  
		  template<class _T>
		  _T operator =	(_T	rhs) { 
			   m = rhs;
			   return rhs;
		  }
   
	};
	
	template<class T>
	struct _reference< _pointer<T, false> >
	{
		  _reference(const	_pointer<T,false>& _m):	 m(_m) 
		  {
		  }
		  _reference(const _reference&	_rm	) :	m(_rm.m)
		  {
		  }
		  _pointer<T,false>	 m;
		  
		  template<class _T>
		  _T operator =	(_T	rhs) { 
			   m = rhs;
			   return rhs;
		  }
   
	};
	  
	   
		 
		 
	   
	struct _reference<Nil>
	{
		inline _reference()	
		{
		}
		inline _reference(const	_reference&	_m)	
		{
		}
		inline _reference(const	Nil& _m) 
		{
		}
	
		   
		inline void	operator=(Nil _m) 
		{
		}
	   
   };					 
	  
	 template<class	T1,	
			  class	T2 = Nil,
			  class	T3 = Nil, 
			  class	T4 = Nil,
			  class	T5 = Nil,
			  class	T6 = Nil >
			 
	struct _lhs
	{
	   _lhs( _reference<T1>	_x1,
			 _reference<T2>	_x2	= _reference<T2>(),
			 _reference<T3>	_x3	= _reference<T3>(),
			 _reference<T4>	_x4	= _reference<T4>(),
			 _reference<T5>	_x5	= _reference<T5>(),
			 _reference<T6>	_x6	= _reference<T6>()):
			 x1(_x1),
			 x2(_x2),
			 x3(_x3),
			 x4(_x4),
			 x5(_x5),
			 x6(_x6)
			 {
			 
			 }
			 
			 template<	class _T1,
						class _T2,
						class _T3,
						class _T4,
						class _T5,
						class _T6>
						  
			_rhs<_T1,_T2,_T3, _T4,_T5, _T6>	operator=(_rhs<_T1,_T2,_T3,_T4,_T5,_T6>	rhs) 
						{
			   x1 =	rhs.x1;
			   x2 =	rhs.x2;
			   x3 =	rhs.x3;
			   x4 =	rhs.x4;
			   x5 =	rhs.x5;
			   x6 =	rhs.x6;
			   return rhs;
			}
		  
		  
	   
		 _reference<T1>	x1;
		 _reference<T2>	x2;
		 _reference<T3>	x3;
		 _reference<T4>	x4;
		 _reference<T5>	x5;
		 _reference<T6>	x6;
		   
	};
	
		
	/* 1 Parameter */
	template<
	  class	T1>
	   _lhs<T1>
				   lhs(	const T1& x1)
					{
					  return _lhs<T1>(_reference<T1>( x1));
					}
	/* 2 Parameters	*/
	  template<
	  class	T1,
	  class	T2>
	   _lhs<T1,	T2>
				   lhs(	const T1& x1,
						const T2& x2)
					{
					  return _lhs<T1,T2>(_reference<T1>( x1	),
										 _reference<T2>( x2	));
					}
					
					  
	/* 3 Parameters	*/
	  template<
	  class	T1,
	  class	T2,
	  class	T3>
	   _lhs<T1,	T2,	T3>
				   lhs(	const T1& x1,
						const T2& x2,
						const T3& x3)
					{
					  return _lhs<T1,T2,T3>( _reference<T3>( x1),
											 _reference<T2>( x2),
											 _reference<T3>( x3));
					}
					
	  /* 4 Parameters */
	  template<
	  class	T1,
	  class	T2,
	  class	T3,
	  class	T4>
	   _lhs<T1,	T2,	T3,	T4>
				   lhs(	const T1& x1,
						const T2& x2,
						const T3& x3,
						const T4& x4)
					{
					  return _lhs<T1,T2,T3,T4>(_reference<T1>(x1),
											   _reference<T2>(x2),
											   _reference<T3>(x3),
											   _reference<T4>(x4) );
					}
	  /* 5 Parameters */
	  template<
	  class	T1,
	  class	T2,
	  class	T3,
	  class	T4,
	  class	T5>
	   _lhs<T1,	T2,	T3,	T4,	T5>
				   lhs(	const T1& x1,
						const T2& x2,
						const T3& x3,
						const T4& x4,
						const T5& x5)
					{
					  return _lhs<T1,T2,T3,T4,T5>(_reference<T1>(x1),
												  _reference<T2>(x2),
												  _reference<T3>(x3),
												  _reference<T4>(x4),
												  _reference<T5>(x5) );
					}
					
					
	  /* 6 Parameters */
	  template<
	  class	T1,
	  class	T2,
	  class	T3,
	  class	T4,
	  class	T5,
	  class	T6>
	   _lhs<T1,	T2,	T3,	T4,	T5,	T6>
				   lhs(	const T1& x1, 
						const T2& x2, 
						const T3& x3, 
						const T4& x4, 
						const T5& x5, 
						const T6& x6)
					{
					  return _lhs<T1,T2,T3,T4,T5,T6>(_reference<T1>(x1),
													 _reference<T2>(x2),
													 _reference<T3>(x3),
													 _reference<T4>(x4),
													 _reference<T5>(x5),
													 _reference<T6>(x6)	);
					}
					
	  
		  
	  template<class T>
		  _out<T, false>   ref(	 T&	a) {
			 return	  _out<T, false> (a) ;
		  }
	  
   
          
BZ_NAMESPACE_END

#endif

/* Usage Example
using namespace blitz;

_rhs<int, Array<int,1> , int>  test() {
          Array<int,1> t(5);
          t = 2;
        
          return blitz::rhs( int(1), t , int(3) 
                               );
  }
  
int main()
{
   int a, b;
   float c = 0;
   Array<int , 1> t(0);
    cout <<  sum(t);
    t = tensor::i;
  
   lhs(a, ref(t) , b ) =     test();
     
   std::cout << "a=" << a << "\n"
             << "b=" << b << "\n"
             << "c=" << c << "\n";
        
   std::cout << "t="<< t;
        
   char k;
   
        
   cin >> k;
}        

*/ 

